
/*
*
*	fdash.c
* --------------------------------------------------------
*  This file contains a number of functions that are peculier to FDASH
*
/

/*
* Standard includes
* --------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

/*
* My includes
* --------------------------------------------------------
*/

#include "fdash.h"
#include "switches_and_constants.c"
#include "stats.h"
#include "structures.c"
#include "random_variates.h"
#include "input_file_handler.h"



/*
**********************************************************
* 				PROTOTYPES
**********************************************************
*/
void print_prog_preamble(void);
void print_out_preamble(char *infile_name, char *outfile_name);
void copy_data(char ***input_file_data, char ***temp_file_data, int total_indivs, int number_cols, int *temp_indivs, char *sample1, char *sample2);
void do_per_locus(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array,char *summary_file_name);
void do_per_sample(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, char *** sub_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array);
void do_pairwise_fst(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, char *** temp_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array);
void do_groups_and_resamples(int permutations, char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names,char **sub_sample_names, char *** temp_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array);
void count_groups(int *number_groups, char ***input_data_array, int total_number_indivs);
char **allocate_group_names_memory(int number_groups,int  max_chars);
void list_groups(char ***input_data_array, int total_number_indivs,char **group_names, int max_number_groups, int *actual_groups);
struct compset *get_mem_for_compared_stats(int number_groups);
void copy_group_data_to_temparray(char *group_name,char *** input_data_array, char ***temp_data_array, int* temp_indivs, int number_loci, int number_indivs);
void count_samples_in_groups(char ***input_data_array,int number_indivs,int number_groups,char **group_names,int *samples_in_group,char **sub_sample_names, int number_samples);
void store_compared_stats(struct statset *incoming, struct compset *out_going);
void calc_test_stat(struct compset *groupstats, struct compset *test_stat, int number_groups);
void shuffle_strings(char **strings,int number);
void shuffle_samples(char **sample_names, char **shuffled_names, int number_samples, char ***input_data_array, int number_indivs);
void empty_significance(struct compset *significance);
void if_stat_greater(struct compset *significance,struct compset *test_stat,struct compset *reference_stat,int permutations);
void print_sig_difs(char *stats_file_name, struct compset *significance);
void print_data_array(char ***input_data_array,int number_indivs,int number_loci);


/*
**********************************************************
* 				FUNCTIONS
**********************************************************
*/



/*
* print_prog_preamble()
* --------------------------------------------------------
*
*	Provides basic user-information to the screen
*/


void print_prog_preamble(void){

printf(	"\t\t***************************");
printf("\n\t\t*    This is F-DASH-ST    *");
printf("\n\t\t***************************");
printf( "\n\nThis program calculates a set of Population-Genetic\nstatistics from allelic phenotype data." );
printf("\n\nNote that this version is compiled to handle a maximum of\n\t%i alleles per locus\n\t%i phenotypes per locus, and \n\t%i alleles per phenotype",TOTAL_MAX_ALLELES, TOTAL_MAX_PHENOTYPES, MAX_ALLELES);
printf("\n\nSignificant differences in statsistics between user-\ndefined groups are calculated by shuffling samples between groups");

}


/*
* print_out_preamble()
* --------------------------------------------------------
*	Timestamps and creates a header for the output file
*/

void print_out_preamble(char *outfile_name, char *infile_name){

FILE *out_stream;
struct tm *current_time;
time_t timer;
time(&timer);

/*get the time*/
current_time=localtime(&timer);
out_stream=fopen(outfile_name,"a");

/*print the preamble*/
fprintf(out_stream,  "***************************");
fprintf(out_stream,"\n*    This is F-DASH-ST    *");
fprintf(out_stream,"\n***************************");
fprintf(out_stream,"\n\n");
fprintf(out_stream,"\nCalculated from %s on %s", infile_name, asctime(current_time));
fclose(out_stream);
}






/*
* copy_data()
* --------------------------------------------------------
* Called for doing Pairwise Fst between samples this copies the data for sample1 and sample2 into a 2D array of strings
*/
void copy_data(char ***input_file_data, char ***temp_file_data, int total_indivs, int number_cols, int *temp_indivs, char *sample1, char *sample2){

	int i,j;

	(*temp_indivs)=0; // need to count the number of lines in the new array
	for (i=0;i<total_indivs;i++){  // for each line in the source array
   	if (			!( strcmp((input_file_data[i])[1],sample2) )		||		!( strcmp((input_file_data[i])[1],sample1) )			){ //if the sample name matches either sample1 or sample2
         for (j=0;j<number_cols;j++){// for each column
         	strcpy((temp_file_data[(*temp_indivs)])[j],(input_file_data[i])[j] );// copy the appropriate string from the source array to the new array
         }
         (*temp_indivs)++;// and increment the number of line in the new array
   	}


   }

}




/*
* do_per_locus()
* --------------------------------------------------------
  Calculate all stats for each locus (accross all samples) and create a weighted average
*/
void do_per_locus(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array,char *summary_file_name){


int i,j ;
int total_indivs_sampled_for_locus;
double *alleles_per_indiv;
FILE *outfilestream;

outfilestream=fopen(stats_file_name,"a");
fprintf(outfilestream,"\n\n\t****Statistics calculated across all samples****\n");
fclose(outfilestream);
print_stat_col_headings(stats_file_name);
alleles_per_indiv=(double*)malloc(number_indivs*sizeof(double));

for (i=ADDITIONAL_COLS;i<number_loci;i++){
	printf("\n\nWorking with locus %s",locus_names[(i-ADDITIONAL_COLS)]);
	take_locus(i,(max_allele_digits+1),number_indivs,input_data_array,locus_phenotypes);

   #ifdef echo_input_file
   for(k=0;k<deme_number;k++){
      printf("\n\n deme %i\n",k);
	   for(j=0;;j++){
      	printf(" %i",(locus_phenotypes[k])[j]);
         if((locus_phenotypes[k])[j]==-1)printf(" ");
         if((locus_phenotypes[k])[j]==-2)break;
      }
      getchar();
   }
   #endif

   get_sample_sizes(i-ADDITIONAL_COLS,deme_number,locus_phenotypes,sample_sizes);
   total_indivs_sampled_for_locus=0;
   for(j=0;j<deme_number;j++)total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];
   printf("\nThe total number of individuals sampled for this locus is %i",total_indivs_sampled_for_locus);

   do_stats(deme_number,sample_sizes[i-ADDITIONAL_COLS],locus_phenotypes,stats_array,i-ADDITIONAL_COLS, total_indivs_sampled_for_locus,alleles_per_indiv,deme_number);
   print_statset(stats_file_name, &(stats_array[i-ADDITIONAL_COLS]),locus_names[i-ADDITIONAL_COLS]);
   print_summary(summary_file_name,locus_phenotypes,sample_sizes[i-ADDITIONAL_COLS],locus_names[i-ADDITIONAL_COLS],deme_number,sample_names);


}
get_weighted_averages(stats_array, sample_sizes,(number_loci-ADDITIONAL_COLS),deme_number);
print_statset(stats_file_name, &(stats_array[number_loci-ADDITIONAL_COLS]),"Average across Loci");
free(alleles_per_indiv);

}



/*
* do_per_sample()
* --------------------------------------------------------
  Calculate all stats for each locus and create a weighted average ( each sample individually)
*/


void do_per_sample(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, char *** sub_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array){

int i,j,deme_start,deme_end, dm, fcounter,total_indivs_sampled_for_locus, sample_number_indivs;
double *alleles_per_indiv;
FILE* outfilestream;


outfilestream=fopen(stats_file_name,"a");
fprintf(outfilestream,"\n\n\t****Statistics calculated for each sample individually****");
fprintf(outfilestream,"\n\n(note that F'st and other inter-sample statistics cannot be calculated for single samples,\nand that overall diversity=within sample diversity for a single sample)");
fclose(outfilestream);

printf("\n\n Doing per-sample stats");
alleles_per_indiv=(double*)malloc(number_indivs*sizeof(double));
deme_end=0;
for (dm=0;dm<deme_number;dm++){
  /*printf("\nThis is sample number %i, indetifier %s",dm,sample_names[dm]);*/
   deme_start=deme_end;
   for (fcounter=deme_end;fcounter<=number_indivs;fcounter++){
      if(fcounter==number_indivs){
      	deme_end=fcounter;
         break;
      }
      if( strcmp((input_data_array[fcounter])[SAMPLE_COL],sample_names[dm])){
	    	deme_end=fcounter;
      	break;
      }
   }
   //printf("\nFind deme_start to be %i an deme_end to be %i",deme_start, deme_end);
   sub_data_array=&(input_data_array[deme_start]);
   sample_number_indivs=deme_end-deme_start;

	outfilestream=fopen(stats_file_name,"a");
	fprintf(outfilestream,"\n\nSample %s", sample_names[dm]);
	fclose(outfilestream);

	print_stat_col_headings(stats_file_name);
	for (i=ADDITIONAL_COLS;i<number_loci;i++){
		//printf("\n\n (per sample) Working with locus %s",locus_names[(i-ADDITIONAL_COLS)]);
		take_locus(i,(max_allele_digits+1),sample_number_indivs,sub_data_array,locus_phenotypes);
   #ifdef echo_input_file

   for(k=0;k<1;k++){
      printf("\n\n deme %i\n",k);
	   for(j=0;;j++){
      	printf(" %i",(locus_phenotypes[k])[j]);
         if((locus_phenotypes[k])[j]==-1)printf(" ");
         if((locus_phenotypes[k])[j]==-2)break;
      }
      getchar();
   }
   #endif

	   get_sample_sizes(i-ADDITIONAL_COLS,1,locus_phenotypes,sample_sizes);
	   total_indivs_sampled_for_locus=0;
	   for(j=0;j<1;j++)total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];
	   do_stats(1,sample_sizes[i-ADDITIONAL_COLS],locus_phenotypes,stats_array,i-ADDITIONAL_COLS, total_indivs_sampled_for_locus,alleles_per_indiv,1);
	   print_statset(stats_file_name, &(stats_array[i-ADDITIONAL_COLS]),locus_names[i-ADDITIONAL_COLS]);
	}
	get_weighted_averages(stats_array, sample_sizes,(number_loci-ADDITIONAL_COLS),1);
	print_statset(stats_file_name, &(stats_array[number_loci-ADDITIONAL_COLS]),"Average across Loci");

} //end for(each deme)
  free(alleles_per_indiv);
} //end function



/*
* do_pairwise_fst()
* --------------------------------------------------------
  Calculate F'st pairwise between all samples
*/

void do_pairwise_fst(char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names, char *** temp_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array){

int inner, outer, i,j, temp_indivs, total_indivs_sampled_for_locus;

double *alleles_per_indiv;

FILE *outfilestream;

outfilestream=fopen(stats_file_name,"a");
fprintf(outfilestream,"\n\n\t****Pairwise F'st (unshared alleles) ****\n\n");

printf("\n\nDoing pairwise F'st");

for (i=0;i<deme_number;i++){fprintf(outfilestream,"\t%s", sample_names[i]);}

for (outer=0;outer<deme_number;outer++){
   fprintf(outfilestream,"\n%s\t",sample_names[outer]);
   for (inner=0;inner<(outer+1);inner++){
   	fprintf(outfilestream,"\t");
   }
	for (inner=(outer+1);inner<deme_number;inner++){
      printf("\n\t%s vs. %s",sample_names[outer],sample_names[inner]);
	   copy_data(input_data_array,temp_data_array,number_indivs,number_loci,&temp_indivs,sample_names[outer], sample_names[inner]);

	   for (i=ADDITIONAL_COLS;i<number_loci;i++){
			take_locus(i,(max_allele_digits+1),temp_indivs,temp_data_array,locus_phenotypes);

		   get_sample_sizes(i-ADDITIONAL_COLS,2,locus_phenotypes,sample_sizes);
		   total_indivs_sampled_for_locus=0;
		   for(j=0;j<2;j++)total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];
		   alleles_per_indiv=(double*)malloc(total_indivs_sampled_for_locus*sizeof(double));
			do_stats(2,sample_sizes[i-ADDITIONAL_COLS],locus_phenotypes,stats_array,i-ADDITIONAL_COLS, total_indivs_sampled_for_locus,alleles_per_indiv,2);

		   free(alleles_per_indiv);
		}
		get_weighted_averages(stats_array, sample_sizes,(number_loci-ADDITIONAL_COLS),2);
   	fprintf(outfilestream,"%f\t",stats_array[number_loci-ADDITIONAL_COLS].fst_thing);

	} //end inner
} //end outer
fclose(outfilestream);
}//end function





//*******************************************************
//************* Group differentiation *********************
//*******************************************************



void do_groups_and_resamples(int permutations, char *stats_file_name, int number_loci, char **locus_names, int max_allele_digits, int number_indivs, char ***input_data_array,
						char **sample_names,char **sub_sample_names, char *** temp_data_array, int **locus_phenotypes, int deme_number, int **sample_sizes, struct statset *stats_array){


int i, j, shfl, rough_number_groups, number_groups, *samples_in_group,grp, temp_indivs,total_indivs_sampled_for_locus;

double*alleles_per_indiv;

struct compset *groupstats, test_stat,reference_stat, significance;
FILE *outfilestream;

char** group_names, **shuffled_names,**groups_in_order;

outfilestream=fopen(stats_file_name,"a");
fprintf(outfilestream,"\n\n\t**** Group Statistics ****\n\n");
printf("\n\nDoing Group Statistics...\n");
fclose(outfilestream);
count_groups(&rough_number_groups,input_data_array,number_indivs);
group_names=allocate_group_names_memory(rough_number_groups, max_allele_digits+1);
list_groups(input_data_array,number_indivs,group_names,rough_number_groups, &number_groups);
shuffled_names=allocate_sample_names_memory(deme_number, max_allele_digits+1);
groups_in_order=allocate_sample_names_memory(deme_number, max_allele_digits+1);

for (i=0;i<deme_number;i++)strcpy(shuffled_names[i],sample_names[i]);

for (i=0;i<deme_number;i++){
	for (j=0;j<number_indivs;j++){
   	if (! strcmp((input_data_array[j])[SAMPLE_COL],sample_names[i])){
          strcpy(		groups_in_order[i],(input_data_array[j])[GROUP_COL]);
      break;
      }
   }
}


samples_in_group=(int *)malloc(number_groups*sizeof(int));
count_samples_in_groups(input_data_array,number_indivs,number_groups,group_names,samples_in_group,sub_sample_names,deme_number);
groupstats=get_mem_for_compared_stats(number_groups);

alleles_per_indiv=(double*)malloc(number_indivs*sizeof(double));
//print_data_array(input_data_array,number_indivs,number_loci);
//unrandomised groups
	for (grp=0;grp<number_groups;grp++){
      outfilestream=fopen(stats_file_name,"a");
		fprintf(outfilestream,"\n\nGroup %s",group_names[grp]);
      fclose(outfilestream);
      printf("\n\tWorking with Group <%s> which comprises %i samples\n", group_names[grp], samples_in_group[grp]);
		copy_group_data_to_temparray(group_names[grp],input_data_array,temp_data_array,&temp_indivs,number_loci,number_indivs);
      print_stat_col_headings(stats_file_name);
   	for (i=ADDITIONAL_COLS;i<number_loci;i++){
			take_locus(i,(max_allele_digits+1),temp_indivs,temp_data_array,locus_phenotypes);
	   	get_sample_sizes(i-ADDITIONAL_COLS,samples_in_group[grp],locus_phenotypes,sample_sizes);
	   	total_indivs_sampled_for_locus=0;
	   	//for(j=0;j<samples_in_group[grp];j++)total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];
	   	for(j=0;j<samples_in_group[grp];j++){total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];}
			do_stats(samples_in_group[grp],sample_sizes[i-ADDITIONAL_COLS],locus_phenotypes,stats_array,i-ADDITIONAL_COLS, total_indivs_sampled_for_locus,alleles_per_indiv,samples_in_group[grp]);
   		print_statset(stats_file_name, &(stats_array[i-ADDITIONAL_COLS]),locus_names[i-ADDITIONAL_COLS]);
	   	//free(alleles_per_indiv);
		}
		get_weighted_averages(stats_array, sample_sizes,(number_loci-ADDITIONAL_COLS),samples_in_group[grp]);
		print_statset(stats_file_name, &(stats_array[number_loci-ADDITIONAL_COLS]),"Average across Loci");
  		store_compared_stats(&(stats_array[number_loci-ADDITIONAL_COLS]),&(groupstats[grp]));
	} //end groups
calc_test_stat(groupstats,&reference_stat,number_groups);

//randomised groups

empty_significance(&significance);
for (shfl=0;shfl<permutations;shfl++){
  	shuffle_samples(groups_in_order,shuffled_names, deme_number,input_data_array,number_indivs);
	printf("\n\tRandomisation: %i",shfl);
   //print_data_array(input_data_array,number_indivs,number_loci);
	for (grp=0;grp<number_groups;grp++){
		copy_group_data_to_temparray(group_names[grp],input_data_array,temp_data_array,&temp_indivs,number_loci,number_indivs);
   	for (i=ADDITIONAL_COLS;i<number_loci;i++){
			take_locus(i,(max_allele_digits+1),temp_indivs,temp_data_array,locus_phenotypes);
	   	get_sample_sizes(i-ADDITIONAL_COLS,samples_in_group[grp],locus_phenotypes,sample_sizes);
	   	total_indivs_sampled_for_locus=0;
	   	for(j=0;j<samples_in_group[grp];j++){total_indivs_sampled_for_locus+= (sample_sizes[i-ADDITIONAL_COLS])[j];}
         do_stats(samples_in_group[grp],sample_sizes[i-ADDITIONAL_COLS],locus_phenotypes,stats_array,i-ADDITIONAL_COLS, total_indivs_sampled_for_locus,alleles_per_indiv,samples_in_group[grp]);
		}
		get_weighted_averages(stats_array, sample_sizes,(number_loci-ADDITIONAL_COLS),samples_in_group[grp]);
  		store_compared_stats(&(stats_array[number_loci-ADDITIONAL_COLS]),&(groupstats[grp]));
	} //end groups

	calc_test_stat(groupstats,&test_stat,number_groups);
	if_stat_greater(&significance,&test_stat,&reference_stat,permutations);
}//end randomisations

print_sig_difs(stats_file_name,&significance);
free(alleles_per_indiv);
free(samples_in_group);

}






/*
* allocate_group_names_memory()
* --------------------------------------------------------
inform the user as the the reading of the file
*/

char **allocate_group_names_memory(int number_groups,int max_chars){

	char **new_mem;
	int i;

   //printf("\n\nAllocating memory for the table of group names...");
	new_mem=(char **)malloc(number_groups*sizeof(char *));
   if (!new_mem){printf("\n\nWARNING memory allocation failed! exiting");getchar();exit(1);}

	for (i=0;i<number_groups;i++){
   	new_mem[i]=malloc(max_chars*sizeof(char *));
      if (!new_mem[i]){printf("\n\nWARNING memory allocation failed! exiting");getchar();exit(1);}
	}
	//printf("\n...success!");
	return new_mem;
}

/*
* count_groups()
* --------------------------------------------------------
count the number of groups in the file
*/
void count_groups(int *number_groups, char ***input_data_array, int total_number_indivs){

int i;
(*number_groups)=1;

for (i=0;i<(total_number_indivs-1);i++){

	if (strcmp(   (input_data_array[i])[GROUP_COL],(input_data_array[i+1])[GROUP_COL]))(*number_groups)++;
}

}


/*
* list_groups()
* --------------------------------------------------------
get the identifying group names
*/

void list_groups(char ***input_data_array, int total_number_indivs,char **group_names, int max_number_groups, int *actual_groups){

int j,i;
(*actual_groups)=0;
for (j=0;j<max_number_groups;j++)(group_names[j])[0]='\0';
for (i=0;i<total_number_indivs;i++){
	for (j=0;j<max_number_groups;j++){
   	if (!strcmp(   (input_data_array[i])[GROUP_COL], group_names[j]   )){ //we've seen it before
      	break;
      }
   	if ( (group_names[j])[0]=='\0'  ){
      	strcpy(group_names[(*actual_groups)++],(input_data_array[i])[GROUP_COL]);
         break;
      }
   }
}



}



/*
* get_mem_for_compared_stats()
* --------------------------------------------------------
claims the memory for the array of stats (one member of each group) that are used to calculate the test statistics.
*/

struct compset *get_mem_for_compared_stats(int number_groups){

	struct compset *new_mem;

   //printf("\n\nAllocating memory for group stats...");
	new_mem=(struct compset *)malloc(number_groups*sizeof(struct compset));
   if (!new_mem){printf("\n\nWARNING memory allocation failed! exiting");getchar();exit(1);}
	//printf("\n...success!");
	return new_mem;



}
/*
* copy_group_data_to_temparray()
* --------------------------------------------------------
copies all those individuals that fall within the current group into the temp data array
*/
void copy_group_data_to_temparray(char *group_name,char *** input_data_array, char ***temp_data_array, int* temp_indivs, int number_loci, int number_indivs){

int i,j;

(*temp_indivs)=0;
for (i=0;i<number_indivs;i++){
	if  (!strcmp(group_name,(input_data_array[i])[GROUP_COL])){
   	for (j=0;j<number_loci;j++){
      	//printf("\nCopying string: %s",(input_data_array[i])[j]);
      	strcpy(  (temp_data_array[*temp_indivs])[j],(input_data_array[i])[j]    );
		}
   (*temp_indivs)++;
   //printf("\n");
	}
}


}


/*
* count_samples_in_groups()
* --------------------------------------------------------
count the number of samples in each group
*/

void count_samples_in_groups(char ***input_data_array,int number_indivs,int number_groups,char **group_names,int *samples_in_group,char **sub_sample_names, int number_samples){

int i,j,x,k;

for (i=0;i<number_groups;i++)samples_in_group[i]=0;
for (i=0;i<number_samples;i++)(sub_sample_names[i])[0]='\0';

x=0;
for (i=0;i<number_indivs;i++){
	for (k=0;k<number_samples;k++){
		if (!   strcmp((input_data_array[i])[SAMPLE_COL],sub_sample_names[k])      ){
      	break;
      }
      if (    (sub_sample_names[k])[0]=='\0'  ){
         strcpy(sub_sample_names[x++],(input_data_array[i])[SAMPLE_COL]);
         for (j=0;j<number_groups;j++){
				if (!   strcmp((input_data_array[i])[GROUP_COL],group_names[j])      ){
            	samples_in_group[j]++;
      			break;
            }
         }
      	break;
      }
   }
}

}//end function





/*
* store_compared_stats()
* --------------------------------------------------------
store the set of stats for the group that we will wish to use in calculating the test stats
*/
void store_compared_stats(struct statset *incoming, struct compset *out_going){

   //alleles and alleles per deme
   out_going->mean_alleles_per_deme=incoming->mean_alleles_per_deme  ;
   //phenotypes and phenotypes per deme
   out_going->mean_phenotypes_per_deme=incoming->mean_phenotypes_per_deme  ;
	//unshared alleles diversity
   out_going->unshared_alleles_within=incoming->unshared_alleles_within  ;
   out_going->unshared_alleles_total=incoming->unshared_alleles_total  ;
   out_going->fst_thing=incoming->fst_thing  ;
	//Phenotypes shannon-weaver diversity
   out_going->phenotypic_shnwv_within=incoming->phenotypic_shnwv_within  ;
   out_going->phenotypic_shnwv_total=incoming->phenotypic_shnwv_total  ;
   out_going->phenotypic_shnwv_fst=incoming->phenotypic_shnwv_fst  ;
	//Phenotypes p(1-p) diversity
   out_going->phenotypic_polymorphism_within=incoming->phenotypic_polymorphism_within  ;
   out_going->phenotypic_polymorphism_total=incoming->phenotypic_polymorphism_total  ;
   out_going->phenotypic_polymorphism_fst=incoming->phenotypic_polymorphism_fst  ;
   // alleles per indiv
   out_going->number_alleles_per_indiv=incoming->number_alleles_per_indiv  ;
   out_going->PseudoLD=incoming->PseudoLD  ;

}


/*
* calc_test_stat()
* --------------------------------------------------------
calculate the test statistic (see help file for FSTAT by J Goudet)
*/
void calc_test_stat(struct compset *groupstats, struct compset *test_stat, int number_groups){

	int i,j;

   //alleles and alleles per deme
   test_stat->mean_alleles_per_deme=0;
   //phenotypes and phenotypes per deme
   test_stat->mean_phenotypes_per_deme=0;
	//unshared alleles diversity
   test_stat->unshared_alleles_within=0;
   test_stat->unshared_alleles_total=0;
   test_stat->fst_thing=0;
	//Phenotypes shannon-weaver diversity
   test_stat->phenotypic_shnwv_within=0;
   test_stat->phenotypic_shnwv_total=0;
   test_stat->phenotypic_shnwv_fst=0;
	//Phenotypes p(1-p) diversity
   test_stat->phenotypic_polymorphism_within=0;
   test_stat->phenotypic_polymorphism_total=0;
   test_stat->phenotypic_polymorphism_fst=0;
   // alleles per indiv
   test_stat->number_alleles_per_indiv=0;
   test_stat->PseudoLD=0;

for (i=0;i<number_groups;i++){
	for (j=(i+1);j<number_groups;j++){
      //test_stat->##=(groupstats[i].##-groupstats[j].##)*(groupstats[i].##-groupstats[j].##);
      test_stat->mean_alleles_per_deme=(groupstats[i].mean_alleles_per_deme-groupstats[j].mean_alleles_per_deme)*(groupstats[i].mean_alleles_per_deme-groupstats[j].mean_alleles_per_deme);
      test_stat->mean_phenotypes_per_deme=(groupstats[i].mean_phenotypes_per_deme-groupstats[j].mean_phenotypes_per_deme)*(groupstats[i].mean_phenotypes_per_deme-groupstats[j].mean_phenotypes_per_deme);
      test_stat->unshared_alleles_within=(groupstats[i].unshared_alleles_within-groupstats[j].unshared_alleles_within)*(groupstats[i].unshared_alleles_within-groupstats[j].unshared_alleles_within);
      test_stat->unshared_alleles_total=(groupstats[i].unshared_alleles_total-groupstats[j].unshared_alleles_total)*(groupstats[i].unshared_alleles_total-groupstats[j].unshared_alleles_total);
      test_stat->fst_thing=(groupstats[i].fst_thing-groupstats[j].fst_thing)*(groupstats[i].fst_thing-groupstats[j].fst_thing);
      test_stat->phenotypic_shnwv_within=(groupstats[i].phenotypic_shnwv_within-groupstats[j].phenotypic_shnwv_within)*(groupstats[i].phenotypic_shnwv_within-groupstats[j].phenotypic_shnwv_within);
      test_stat->phenotypic_shnwv_total=(groupstats[i].phenotypic_shnwv_total-groupstats[j].phenotypic_shnwv_total)*(groupstats[i].phenotypic_shnwv_total-groupstats[j].phenotypic_shnwv_total);
      test_stat->phenotypic_shnwv_fst=(groupstats[i].phenotypic_shnwv_fst-groupstats[j].phenotypic_shnwv_fst)*(groupstats[i].phenotypic_shnwv_fst-groupstats[j].phenotypic_shnwv_fst);
      test_stat->phenotypic_polymorphism_within=(groupstats[i].phenotypic_polymorphism_within-groupstats[j].phenotypic_polymorphism_within)*(groupstats[i].phenotypic_polymorphism_within-groupstats[j].phenotypic_polymorphism_within);
      test_stat->phenotypic_polymorphism_total=(groupstats[i].phenotypic_polymorphism_total-groupstats[j].phenotypic_polymorphism_total)*(groupstats[i].phenotypic_polymorphism_total-groupstats[j].phenotypic_polymorphism_total);
      test_stat->phenotypic_polymorphism_fst=(groupstats[i].phenotypic_polymorphism_fst-groupstats[j].phenotypic_polymorphism_fst)*(groupstats[i].phenotypic_polymorphism_fst-groupstats[j].phenotypic_polymorphism_fst);
      test_stat->number_alleles_per_indiv=(groupstats[i].number_alleles_per_indiv-groupstats[j].number_alleles_per_indiv)*(groupstats[i].number_alleles_per_indiv-groupstats[j].number_alleles_per_indiv);
      test_stat->PseudoLD=(groupstats[i].PseudoLD-groupstats[j].PseudoLD)*(groupstats[i].PseudoLD-groupstats[j].PseudoLD);
   }
}
}


/*
* shuffle_samples()
* --------------------------------------------------------
reasign individuals to new samples
ie. all individuals currently in sample alpha will go to sample beta, the group assignments are unchanged, so effect is to shuffle samples between groups
*/


void shuffle_samples(char **groups_in_order, char **shuffled_names, int number_samples, char ***input_data_array, int number_indivs){

	int i,j;

	shuffle_strings(shuffled_names,number_samples);
  // for (i=0;i<number_samples;i++)printf("\n%20s\t %20s",sample_names[i],shuffled_names[i]);getchar();

	for (i=0;i<number_indivs;i++){
   	for (j=0;j<number_samples;j++){
      	//printf("\nComparing %s with %s",sample_names[j],(input_data_array[i])[SAMPLE_COL]);
      	if (!strcmp(shuffled_names[j],(input_data_array[i])[SAMPLE_COL])){
	      	//printf("\n\t thus overwriting %s with %s",(input_data_array[i])[SAMPLE_COL],shuffled_names[j]);
            strcpy((input_data_array[i])[GROUP_COL],groups_in_order[j]);
            break; // placed here after 2 hours of bug-hunting 12:14 21/01/2004
         }
      }
   //getchar();
	}

}


/*
* shuffle_samples()
* --------------------------------------------------------
Shuffle an array of strings
*/
void shuffle_strings(char **strings,int number){

  char temp_string[500]; //lets hope no-one has a population name >499 characters
  int new_int,i;

//  printf("\nShuffling!");

  for (i=0;i<(number-1);i++){
      new_int=i+(int)( (number-i)*random_0to1());
      strcpy(temp_string,strings[i]);
//      printf("\nswapping %s with %s",strings[i],strings[new_int]);
		strcpy(strings[i],strings[new_int]);
      strcpy(strings[new_int],temp_string);
  }

}


/*
* empty_significance()
* --------------------------------------------------------
set all the values to zero
*/
void empty_significance(struct compset *significance){

   //alleles and alleles per deme
   significance->mean_alleles_per_deme=0;
   //phenotypes and phenotypes per deme
   significance->mean_phenotypes_per_deme=0;
	//unshared alleles diversity
   significance->unshared_alleles_within=0;
   significance->unshared_alleles_total=0;
   significance->fst_thing=0;
	//Phenotypes shannon-weaver diversity
   significance->phenotypic_shnwv_within=0;
   significance->phenotypic_shnwv_total=0;
   significance->phenotypic_shnwv_fst=0;
	//Phenotypes p(1-p) diversity
   significance->phenotypic_polymorphism_within=0;
   significance->phenotypic_polymorphism_total=0;
   significance->phenotypic_polymorphism_fst=0;
   // alleles per indiv
   significance->number_alleles_per_indiv=0;
   significance->PseudoLD=0;
}
/*
* if_stat_greater()
* --------------------------------------------------------
record the proportion or randomisations that have a value greater than the reference statistics
*/

void if_stat_greater(struct compset *significance,struct compset *test_stat,struct compset *reference_stat,int permutations){

   //alleles and alleles per deme
   if(test_stat->mean_alleles_per_deme>reference_stat->mean_alleles_per_deme)significance->mean_alleles_per_deme+=(double)1.0/permutations;
   //phenotypes and phenotypes per deme
   if(test_stat->mean_phenotypes_per_deme>reference_stat->mean_phenotypes_per_deme)significance->mean_phenotypes_per_deme+=(double)1.0/permutations;
	//unshared alleles diversity
   if(test_stat->unshared_alleles_within>reference_stat->unshared_alleles_within)significance->unshared_alleles_within+=(double)1.0/permutations;
   if(test_stat->unshared_alleles_total>reference_stat->unshared_alleles_total)significance->unshared_alleles_total+=(double)1.0/permutations;
   if(test_stat->fst_thing>reference_stat->fst_thing)significance->fst_thing+=(double)1.0/permutations;
	//Phenotypes shannon-weaver diversity
   if(test_stat->phenotypic_shnwv_within>reference_stat->phenotypic_shnwv_within)significance->phenotypic_shnwv_within+=(double)1.0/permutations;
   if(test_stat->phenotypic_shnwv_total>reference_stat->phenotypic_shnwv_total)significance->phenotypic_shnwv_total+=(double)1.0/permutations;
   if(test_stat->phenotypic_shnwv_fst>reference_stat->phenotypic_shnwv_fst)significance->phenotypic_shnwv_fst+=(double)1.0/permutations;
	//Phenotypes p(1-p) diversity
   if(test_stat->phenotypic_polymorphism_within>reference_stat->phenotypic_polymorphism_within)significance->phenotypic_polymorphism_within+=(double)1.0/permutations;
   if(test_stat->phenotypic_polymorphism_total>reference_stat->phenotypic_polymorphism_total)significance->phenotypic_polymorphism_total+=(double)1.0/permutations;
   if(test_stat->phenotypic_polymorphism_fst>reference_stat->phenotypic_polymorphism_fst)significance->phenotypic_polymorphism_fst+=(double)1.0/permutations;
   // alleles per indiv
   if(test_stat->number_alleles_per_indiv>reference_stat->number_alleles_per_indiv)significance->number_alleles_per_indiv+=(double)1.0/permutations;
   if(test_stat->PseudoLD>reference_stat->PseudoLD)significance->PseudoLD+=(double)1.0/permutations;

}


/*
* print_sig_difs()
* --------------------------------------------------------
printout the p values for sig difs

*/
void print_sig_difs(char *stats_file_name, struct compset *significance){

FILE *output_stream;
output_stream=fopen(stats_file_name,"a");

   fprintf(output_stream,"\n\n\nSignificant Differences between Groups");
   fprintf(output_stream,"\n\nP-values based on the proportion or randomised samples that give a test statistic greater than the observed one ");
   fprintf(output_stream,"\n\n");

   fprintf(output_stream,"\tAv. number of alleles per sample");
   fprintf(output_stream,"\tAv. number of phenotypes per sample");

   fprintf(output_stream,"\tAv. number of unshared alleles between pairs of individuals within samples");
   fprintf(output_stream,"\tAv. number of unshared alleles between pairs of individuals across all samples");
   fprintf(output_stream,"\t(unshared alleles) F'st");

   fprintf(output_stream,"\tAv. Shannon-Weaver phenotype diversity within samples");
   fprintf(output_stream,"\tShannon-Weaver phenotype diversity accross all samples");
   fprintf(output_stream,"\t(Shannon-Weaver) F'st");

   fprintf(output_stream,"\tAv. p(1-p) phenotype diversity within samples");
   fprintf(output_stream,"\tp(1-p) phenotype diversity accross all samples");
   fprintf(output_stream,"\t(p(1-p)) F'st");

   fprintf(output_stream,"\tAverage number of different alleles carried by each individual");
   fprintf(output_stream,"\tPsLD");

   fprintf(output_stream,"\n");

   fprintf(output_stream,"\t%1.5f",(double)significance->mean_alleles_per_deme);
   fprintf(output_stream,"\t%1.5f",(double)significance->mean_phenotypes_per_deme);

   fprintf(output_stream,"\t%1.5f",(double)significance->unshared_alleles_within);
   fprintf(output_stream,"\t%1.5f",(double)significance->unshared_alleles_total);
   fprintf(output_stream,"\t%1.5f",(double)significance->fst_thing);

   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_shnwv_within);
   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_shnwv_total);
   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_shnwv_fst);

   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_polymorphism_within);
   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_polymorphism_total);
   fprintf(output_stream,"\t%1.5f",(double)significance->phenotypic_polymorphism_fst);


   fprintf(output_stream,"\t%1.5f",(double)significance->number_alleles_per_indiv);
   fprintf(output_stream,"\t%1.5f",(double)significance->PseudoLD);

   fclose (output_stream);
}
















void print_data_array(char ***input_data_array,int number_indivs,int number_loci){

	int i,j,k;

 for (i=0;i<number_indivs;i++){
   printf("\n%3i   ",i);
 	for (j=0;j<number_loci;j++){
      printf(" ");
	 	for (k=0;k<8;k++){
      	if (  ((input_data_array[i])[j])[k]=='\0')break;
      	printf("%c",((input_data_array[i])[j])[k]);
      }
   }
 }
 getchar();
}

