/*
*
*	structures.c
* --------------------------------------------------------
*
* This contains all the universal structures used in the code
* It also contains usefull globals
*
*
*/
#ifndef _structures_h
#define _structures_h

#include "switches_and_constants.c"

/*				STRUCTURES
* --------------------------------------------------------
*/

/*
* node
* --
* this is the structure around which all our trees are based, it holds:
* 	time 				(a double representing the time in units of 2N generations until the next coalescent event)
*  id					an identifying number, mainly to make debugging easier
*  genome			which genome this node is in;
*	mutations		int number of mutations on the branch between this node and its ancestor. (placed on the tree after the topology is built.
*	allele			which allele (infinite alleles) this individual carries.
*	descendant1 	pointer to the left descendant
* 	descendant2 	pointer to the right descendant
* 	ancestor 		pointer to the ancestor
*/


struct node {
	double time;
   int id;
   int genome;
   int mutations;
   int allele;
   struct node *descendant1;
   struct node *descendant2;
   struct node *ancestor;
};

/*
* statset
* --
* Structure used to form an array of stats for all the repeats
*/

struct statset {

   //alleles and alleles per deme
	double total_number_alleles;
   double mean_alleles_per_deme;
   double *total_alleles_per_deme;

   //phenotypes and phenotypes per deme
   double total_number_phenotypes;
   double mean_phenotypes_per_deme;
   double *total_phenotypes_per_deme;

	//unshared alleles diversity
   double unshared_alleles_within;
   double unshared_alleles_total;
   double fst_thing;

	//Phenotypes shannon-weaver diversity
   double phenotypic_shnwv_within;
   double phenotypic_shnwv_total;
   double phenotypic_shnwv_fst;

	//Phenotypes p(1-p) diversity
   double phenotypic_polymorphism_within;
   double phenotypic_polymorphism_total;
   double phenotypic_polymorphism_fst;

   // alleles per indiv
   double number_alleles_per_indiv;
   double var_of_alleles_per_indiv;
   double max_alleles_per_indiv;
   double min_alleles_per_indiv;

   //alleles per genotype
   double average_bands_per_phenotype;
   double varience_of_bands_per_phenotype;

	//other allele-sharing stats
   double shared_alleles_within;
   double shared_alleles_between;
   double unshared_alleles_between;
   double shared_alleles_total;

   // %of indivs carrying each allele stats
   double *allele_freq_distribution;
   double *per_deme_allele_freq_distribution;

   double mean_allele_carriers_overall;
   double var_allele_carriers_overall;
	double skew_allele_carriers_overall;
   double frequ_commonest_allele_overall;
   double frequ_rarest_allele_overall;

   double mean_allele_carriers_within;
   double var_allele_carriers_within;
   double skew_allele_carriers_within;
   double frequ_commonest_allele_within;
   double frequ_rarest_allele_within;



	// %of indivs carrying each phenotype stats
   double *phenotype_freq_distribution;
   double *per_deme_phenotype_freq_distribution;

   double mean_phenotype_carriers_overall;
   double var_phenotype_carriers_overall;
	double skew_phenotype_carriers_overall;
   double frequ_commonest_phenotype_overall;
   double frequ_rarest_phenotype_overall;

   double mean_phenotype_carriers_within;
   double var_phenotype_carriers_within;
   double skew_phenotype_carriers_within;
   double frequ_commonest_phenotype_within;
   double frequ_rarest_phenotype_within;

   // correlation between alleles
   double PseudoLD;
   double calculable_PseudoLD;

   //Locus polymorphism
   double polymorphic_locus;  // is there more than one phenotype present?




   // the number of isoloci each allele appears in
   double isoloci_per_allele;

   };

/*
* compset
* --
* Structure used to form an array of stats that will be compared between groups by a resampling procedure
*/

struct compset {

   //alleles and alleles per deme
   double mean_alleles_per_deme;
   //phenotypes and phenotypes per deme
   double mean_phenotypes_per_deme;
	//unshared alleles diversity
   double unshared_alleles_within;
   double unshared_alleles_total;
   double fst_thing;
	//Phenotypes shannon-weaver diversity
   double phenotypic_shnwv_within;
   double phenotypic_shnwv_total;
   double phenotypic_shnwv_fst;
	//Phenotypes p(1-p) diversity
   double phenotypic_polymorphism_within;
   double phenotypic_polymorphism_total;
   double phenotypic_polymorphism_fst;
   // alleles per indiv
   double number_alleles_per_indiv;
   double PseudoLD;
   };


#endif